@@ -1,3 +1,22 @@
+/* 
+  Copyright(C) 2010 Tetsuro IKEDA
+  Copyright(C) 2010 Kentoku SHIBA
+
+  This library is free software; you can redistribute it and/or
+  modify it under the terms of the GNU Lesser General Public
+  License as published by the Free Software Foundation; either
+  version 2.1 of the License, or (at your option) any later version.
+
+  This library is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+  Lesser General Public License for more details.
+
+  You should have received a copy of the GNU Lesser General Public
+  License along with this library; if not, write to the Free Software
+  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+*/
+
 #ifdef USE_PRAGMA_IMPLEMENTATION
 #pragma implementation
 #endif
@@ -60,6 +79,95 @@ struct st_mysql_sys_var *mrn_system_variables[] =
   NULL
 };
 
+
+/* Groonga information schema */
+int GROONGA_VERSION_SHORT = 0x0001;
+static const char plugin_author[] = "Yoshinori Matsunobu";
+static struct st_mysql_information_schema	i_s_info =
+{
+	MYSQL_INFORMATION_SCHEMA_INTERFACE_VERSION
+};
+
+static ST_FIELD_INFO	i_s_groonga_stats_fields_info[] =
+{
+  {
+    "VERSION",
+    40,
+    MYSQL_TYPE_STRING,
+    0,
+    0,
+    "",
+    SKIP_OPEN_TABLE
+  },
+  {
+    "rows_written",
+    MY_INT32_NUM_DECIMAL_DIGITS,
+    MYSQL_TYPE_LONG,
+    0,
+    0,
+    "Rows written to groonga",
+    SKIP_OPEN_TABLE
+  },
+  {
+    "rows_read",
+    MY_INT32_NUM_DECIMAL_DIGITS,
+    MYSQL_TYPE_LONG,
+    0,
+    0,
+    "Rows read from groonga",
+    SKIP_OPEN_TABLE
+  }
+};
+
+static int i_s_groonga_stats_deinit(void* p)
+{
+  DBUG_ENTER("i_s_common_deinit");
+  DBUG_RETURN(0);
+}
+
+static int i_s_groonga_stats_fill(
+  THD* thd, TABLE_LIST* tables, COND* cond)
+{
+  TABLE*	table	= (TABLE *) tables->table;
+  int	status	= 0;
+  DBUG_ENTER("i_s_groonga_fill_low");
+  table->field[0]->store(grn_get_version(), strlen(grn_get_version()),
+     system_charset_info);
+  table->field[0]->set_notnull();
+  table->field[1]->store(1); /* TODO */
+  table->field[2]->store(2); /* TODO */
+  if (schema_table_store_record(thd, table)) {
+    status = 1;
+  }
+  DBUG_RETURN(status);
+}
+
+static int i_s_groonga_stats_init(void* p)
+{
+  DBUG_ENTER("i_s_groonga_stats_init");
+  ST_SCHEMA_TABLE* schema = (ST_SCHEMA_TABLE*) p;
+  schema->fields_info = i_s_groonga_stats_fields_info;
+  schema->fill_table = i_s_groonga_stats_fill;
+  DBUG_RETURN(0);
+}
+
+struct st_mysql_plugin i_s_groonga_stats =
+{
+  MYSQL_INFORMATION_SCHEMA_PLUGIN,
+  &i_s_info,
+  "groonga_stats",
+  plugin_author,
+  "Statistics for groonga",
+  0,
+  i_s_groonga_stats_init,
+  i_s_groonga_stats_deinit,
+  0x0001,
+  NULL,
+  NULL,
+  NULL,
+};
+/* End of groonga information schema implementations */
+
 mysql_declare_plugin(mroonga)
 {
   MYSQL_STORAGE_ENGINE_PLUGIN,
@@ -67,14 +175,14 @@ mysql_declare_plugin(mroonga)
   "groonga",
   "Tetsuro IKEDA",
   "Fulltext search, column base",
-  PLUGIN_LICENSE_BSD,
+  0,
   mrn_init,
   mrn_deinit,
   0x0001,
   mrn_status_variables,
   mrn_system_variables,
   NULL
-}
+},i_s_groonga_stats
 mysql_declare_plugin_end;
 
 int mrn_init(void *p)
@@ -486,7 +594,7 @@ ha_mroonga::~ha_mroonga()
 const char *ha_mroonga::table_type() const
 {
   DBUG_ENTER("ha_mroonga::table_type");
-  DBUG_RETURN("mroonga");
+  DBUG_RETURN("groonga");
 }
 
 const char *ha_mroonga::index_type(uint keynr)
@@ -521,6 +629,8 @@ ulonglong ha_mroonga_table_flags =
     HA_NO_PREFIX_CHAR_KEYS |
     HA_CAN_FULLTEXT |
     HA_NO_AUTO_INCREMENT |
+    HA_CAN_INSERT_DELAYED |
+    HA_BINLOG_FLAGS |
     HA_CAN_BIT_FIELD;
     //HA_HAS_RECORDS;
 
